; DMGTRIS
; Copyright (C) 2023 - Randy Thiemann <randy.thiemann@gmail.com>

; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <https://www.gnu.org/licenses/>.


IF !DEF(TILES_INC)
DEF TILES_INC EQU 1


INCLUDE "globals.asm"


SECTION "Static Global data", ROMX, BANK[BANK_OTHER]
; Speed curve data is defined as follows:
; N blocks of:
;   dw BCD_START_LEVEL, START_LEVEL, BCD_NEXT_100_LEVEL_BREAKPOINT
;   db GRID_CELLS_PER_MOVE_ON_OVERFLOW, INCREMENT_PER_FRAME (0 means overflow each frame)
;   db NORMAL_ARE, LINE_ARE, DAS, LOCK_DELAY, CLEAR_DELAY
;
; Followed by one single:
;   dw $FFFF
sDMGTSpeedCurve::       ; Speed curve of the game.
    dw $0000, 0, $0100  ; Level 0000
    db 1, 16
    db 25, 15, 14, 30, 20

    dw $0015, 15, $0100 ; Level 0015
    db 1, 17
    db 25, 15, 14, 30, 20

    dw $0030, 30, $0100 ; Level 0030
    db 1, 18
    db 25, 15, 14, 30, 20

    dw $0040, 40, $0100 ; Level 0040
    db 1, 20
    db 25, 15, 14, 30, 20

    dw $0050, 50, $0100 ; Level 0050
    db 1, 21
    db 25, 15, 14, 30, 20

    dw $0060, 60, $0100 ; Level 0060
    db 1, 23
    db 25, 15, 14, 30, 20

    dw $0070, 70, $0100 ; Level 0070
    db 1, 26
    db 25, 15, 14, 30, 20

    dw $0080, 80, $0100 ; Level 0080
    db 1, 28
    db 25, 15, 14, 30, 20

    dw $0090, 90, $0100 ; Level 0090
    db 1, 32
    db 25, 15, 14, 30, 20

    dw $0100, 100, $0200 ; Level 0100
    db 1, 37
    db 25, 15, 14, 30, 20

    dw $0150, 150, $0200 ; Level 0150
    db 1, 43
    db 25, 15, 14, 30, 20

    dw $0200, 200, $0300 ; Level 0200
    db 1, 51
    db 25, 15, 14, 30, 20

    dw $0225, 225, $0300 ; Level 0225
    db 1, 64
    db 25, 15, 14, 30, 20

    dw $0250, 250, $0300 ; Level 0250
    db 1, 85
    db 25, 15, 14, 30, 20

    dw $0275, 275, $0300 ; Level 0275
    db 1, 128
    db 25, 15, 14, 30, 20

    dw $0300, 300, $0400 ; Level 0300
    db 1, $00
    db 25, 7, 14, 30, 20

    dw $0350, 350, $0350 ; Level 0350
    db 2, $00
    db 25, 7, 14, 30, 20

    dw $0400, 400, $0400 ; Level 0400
    db 3, $00
    db 25, 7, 14, 30, 20

    dw $0450, 450, $0500 ; Level 0450
    db 4, $00
    db 25, 7, 14, 30, 20

    dw $0475, 475, $0500 ; Level 0475
    db 5, $00
    db 25, 7, 14, 30, 20

    dw $0500, 500, $0600 ; Level 0500
    db 20, $00
    db 25, 6, 14, 30, 20

    dw $0600, 600, $0700 ; Level 0600
    db 20, $00
    db 25, 6, 8, 30, 20

    dw $0700, 700, $0800 ; Level 0700
    db 20, $00
    db 20, 6, 8, 30, 20

    dw $0900, 900, $1000 ; Level 0900
    db 20, $00
    db 16, 4, 6, 25, 16

    dw $1100, 1100, $1200 ; Level 1100
    db 20, $00
    db 12, 4, 6, 25, 16

    dw $1200, 1200, $1300 ; Level 1200
    db 20, $00
    db 12, 4, 6, 25, 8

    dw $1300, 1300, $1400 ; Level 1300
    db 20, $00
    db 10, 4, 6, 20, 7

    dw $1400, 1400, $1500 ; Level 1400
    db 20, $00
    db 10, 4, 6, 18, 6

    dw $1500, 1500, $1600 ; Level 1500
    db 20, $00
    db 8, 4, 4, 16, 5

    dw $1600, 1600, $1700 ; Level 1600
    db 20, $00
    db 8, 4, 4, 14, 4

    dw $1700, 1700, $1800 ; Level 1700
    db 20, $00
    db 6, 4, 4, 12, 3

    dw $1800, 1800, $1900 ; Level 1800
    db 20, $00
    db 6, 4, 4, 10, 3

    dw $1900, 1900, $2000 ; Level 1900
    db 20, $00
    db 4, 4, 4, 8, 3

    dw $2000, 2000, $2100 ; Level 2000
    db 20, $00
    db 4, 4, 3, 8, 3

    dw $2500, 2500, $2600 ; Level 2500
    db 20, $00
    db 2, 2, 1, 8, 2

    dw $3000, 3000, $3100 ; Level 3000
    db 20, $00
    db 1, 1, 1, 8, 1

    dw $4000, 4000, $4100 ; Level 4000
    db 20, $00
    db 1, 1, 1, 6, 1

    dw $5000, 5000, $5100 ; Level 5000
    db 20, $00
    db 1, 1, 1, 4, 1

    dw $6666, 6666, $6700 ; Level 6666
    db 20, $00
    db 1, 1, 1, 3, 1

sDMGTSpeedCurveEnd::
    dw $FFFF              ; End.

sDMGTSpeedCurveSpecialData::
    dw 2000     ; When do we get bones?
    dw $FFFF    ; When do blocks turn invisible?
    dw 9999     ; When is the hard kill screen?
    dw $9999    ; Same but in BCD.
    dw 9998     ; If the last level lock isn't at x99, where is it?
    dw 3600     ; How long does the staff roll last after the kill screen, in frames.
    db $FF      ; Big mode in staff roll?


sTGM1SpeedCurve::
    dw $0000, 0, $0100
    db 1, 4
    db 30, 30, 16, 30, 41

    dw $0030, 30, $0100
    db 1, 6
    db 30, 30, 16, 30, 41

    dw $0035, 35, $0100
    db 1, 8
    db 30, 30, 16, 30, 41

    dw $0040, 40, $0100
    db 1, 10
    db 30, 30, 16, 30, 41

    dw $0050, 50, $0100
    db 1, 12
    db 30, 30, 16, 30, 41

    dw $0060, 60, $0100
    db 1, 16
    db 30, 30, 16, 30, 41

    dw $0070, 70, $0100
    db 1, 32
    db 30, 30, 16, 30, 41

    dw $0080, 80, $0100
    db 1, 48
    db 30, 30, 16, 30, 41

    dw $0090, 90, $0100
    db 1, 64
    db 30, 30, 16, 30, 41

    dw $0100, 100, $0200
    db 1, 80
    db 30, 30, 16, 30, 41

    dw $0120, 120, $0200
    db 1, 96
    db 30, 30, 16, 30, 41

    dw $0140, 140, $0200
    db 1, 112
    db 30, 30, 16, 30, 41

    dw $0160, 160, $0200
    db 1, 128
    db 30, 30, 16, 30, 41

    dw $0170, 170, $0200
    db 1, 144
    db 30, 30, 16, 30, 41

    dw $0200, 200, $0300
    db 1, 4
    db 30, 30, 16, 30, 41

    dw $0220, 220, $0300
    db 1, 32
    db 30, 30, 16, 30, 41

    dw $0230, 230, $0300
    db 1, 64
    db 30, 30, 16, 30, 41

    dw $0233, 233, $0300
    db 1, 96
    db 30, 30, 16, 30, 41

    dw $0236, 236, $0300
    db 1, 128
    db 30, 30, 16, 30, 41

    dw $0239, 239, $0300
    db 1, 160
    db 30, 30, 16, 30, 41

    dw $0243, 243, $0300
    db 1, 192
    db 30, 30, 16, 30, 41

    dw $0247, 247, $0300
    db 1, 224
    db 30, 30, 16, 30, 41

    dw $0251, 251, $0300
    db 1, $00
    db 30, 30, 16, 30, 41

    dw $0300, 300, $0400
    db 2, $00
    db 30, 30, 16, 30, 41

    dw $0330, 330, $0400
    db 3, $00
    db 30, 30, 16, 30, 41

    dw $0360, 360, $0400
    db 4, $00
    db 30, 30, 16, 30, 41

    dw $0400, 400, $0500
    db 5, $00
    db 30, 30, 16, 30, 41

    dw $0420, 420, $0500
    db 4, $00
    db 30, 30, 16, 30, 41

    dw $0450, 450, $0500
    db 3, $00
    db 30, 30, 16, 30, 41

    dw $0500, 500, $0600
    db 20, $00
    db 30, 30, 16, 30, 41

    dw $0600, 600, $0700
    db 20, $00
    db 30, 30, 16, 30, 41

    dw $0700, 700, $0800
    db 20, $00
    db 30, 30, 16, 30, 41

    dw $0800, 800, $0900
    db 20, $00
    db 30, 30, 16, 30, 41

    dw $0900, 900, $0999
    db 20, $00
    db 30, 30, 16, 30, 41

    dw $0990, 990, $0999
    db 20, $00
    db 30, 30, 16, 30, 41

sTGM1SpeedCurveEnd::
    dw $FFFF

sTGM1SpeedCurveSpecialData::
    dw 999      ; When do we get bones?
    dw $FFFF    ; When do blocks turn invisible?
    dw 999      ; When is the hard kill screen?
    dw $0999    ; Same but in BCD.
    dw 998      ; If the last level lock isn't at x99, where is it?
    dw 3600     ; How long does the staff roll last after the kill screen, in frames.
    db $FF      ; Big mode in staff roll?


sCHILSpeedCurve::
    dw $0000, 0, $0100
    db 1, 5
    db 10, 10, 16, 25, 17

    dw $0010, 10, $0100
    db 1, 6
    db 10, 10, 16, 25, 17

    dw $0020, 20, $0100
    db 1, 7
    db 10, 10, 16, 25, 17

    dw $0030, 30, $0100
    db 1, 8
    db 10, 10, 16, 25, 17

    dw $0040, 40, $0100
    db 1, 9
    db 10, 10, 16, 25, 17

    dw $0050, 50, $0100
    db 1, 11
    db 10, 10, 16, 25, 17

    dw $0060, 60, $0100
    db 1, 14
    db 10, 10, 16, 25, 17

    dw $0070, 70, $0100
    db 1, 20
    db 10, 10, 16, 25, 17

    dw $0080, 80, $0100
    db 1, 32
    db 10, 10, 16, 25, 17

    dw $0090, 90, $0100
    db 1, 43
    db 10, 10, 16, 25, 17

    dw $0100, 100, $0200
    db 1, 51
    db 10, 10, 16, 25, 17

    dw $0130, 130, $0200
    db 1, 64
    db 10, 10, 16, 25, 17

    dw $0160, 160, $0200
    db 1, 85
    db 10, 10, 16, 25, 17

    dw $0190, 190, $0200
    db 1, 128
    db 10, 10, 16, 25, 17

    dw $0290, 290, $0300
    db 1, $00
    db 10, 10, 16, 25, 17

    dw $0300, 300, $0400
    db 2, $00
    db 10, 10, 12, 25, 17

    dw $0400, 400, $0500
    db 3, $00
    db 10, 10, 12, 25, 17

    dw $0500, 500, $0600
    db 4, $00
    db 10, 10, 12, 25, 17

    dw $0600, 600, $0700
    db 5, $00
    db 10, 10, 12, 25, 17

    dw $1000, 1000, $1100
    db 20, $00
    db 10, 10, 8, 25, 17

    dw $2500, 2500, $2600
    db 20, $00
    db 10, 10, 6, 18, 17

    dw $5000, 5000, $5100
    db 20, $00
    db 5, 5, 6, 14, 10

sCHILSpeedCurveEnd::
    dw $FFFF

sCHILSpeedCurveSpecialData::
    dw $FFFF    ; When do we get bones?
    dw $FFFF    ; When do blocks turn invisible?
    dw 9999     ; When is the hard kill screen?
    dw $9999    ; Same but in BCD.
    dw 9998     ; If the last level lock isn't at x99, where is it?
    dw $FFFF    ; How long does the staff roll last after the kill screen, in frames.
    db $00      ; Big mode in staff roll?


sTGM3SpeedCurve::
    dw $0000, 0, $0100
    db 1, 4
    db 27, 27, 16, 30, 40

    dw $0030, 30, $0100
    db 1, 6
    db 27, 27, 16, 30, 40

    dw $0035, 35, $0100
    db 1, 8
    db 27, 27, 16, 30, 40

    dw $0040, 40, $0100
    db 1, 10
    db 27, 27, 16, 30, 40

    dw $0050, 50, $0100
    db 1, 12
    db 27, 27, 16, 30, 40

    dw $0060, 60, $0100
    db 1, 16
    db 27, 27, 16, 30, 40

    dw $0070, 70, $0100
    db 1, 32
    db 27, 27, 16, 30, 40

    dw $0080, 80, $0100
    db 1, 48
    db 27, 27, 16, 30, 40

    dw $0090, 90, $0100
    db 1, 64
    db 27, 27, 16, 30, 40

    dw $0100, 100, $0200
    db 1, 80
    db 27, 27, 16, 30, 40

    dw $0120, 120, $0200
    db 1, 96
    db 27, 27, 16, 30, 40

    dw $0140, 140, $0200
    db 1, 112
    db 27, 27, 16, 30, 40

    dw $0160, 160, $0200
    db 1, 128
    db 27, 27, 16, 30, 40

    dw $0170, 170, $0200
    db 1, 144
    db 27, 27, 16, 30, 40

    dw $0200, 200, $0300
    db 1, 4
    db 27, 27, 16, 30, 40

    dw $0220, 220, $0300
    db 1, 32
    db 27, 27, 16, 30, 40

    dw $0230, 230, $0300
    db 1, 64
    db 27, 27, 16, 30, 40

    dw $0233, 233, $0300
    db 1, 96
    db 27, 27, 16, 30, 40

    dw $0236, 236, $0300
    db 1, 128
    db 27, 27, 16, 30, 40

    dw $0239, 239, $0300
    db 1, 160
    db 27, 27, 16, 30, 40

    dw $0243, 243, $0300
    db 1, 192
    db 27, 27, 16, 30, 40

    dw $0247, 247, $0300
    db 1, 224
    db 27, 27, 16, 30, 40

    dw $0251, 251, $0300
    db 1, $00
    db 27, 27, 16, 30, 40

    dw $0300, 300, $0400
    db 2, $00
    db 27, 27, 16, 30, 40

    dw $0330, 330, $0400
    db 3, $00
    db 27, 27, 16, 30, 40

    dw $0360, 360, $0400
    db 4, $00
    db 27, 27, 16, 30, 40

    dw $0400, 400, $0500
    db 5, $00
    db 27, 27, 16, 30, 40

    dw $0420, 420, $0500
    db 4, $00
    db 27, 27, 16, 30, 40

    dw $0450, 450, $0500
    db 3, $00
    db 27, 27, 16, 30, 40

    dw $0500, 500, $0600
    db 20, $00
    db 27, 27, 10, 30, 25

    dw $0600, 600, $0700
    db 20, $00
    db 27, 18, 10, 30, 16

    dw $0700, 700, $0800
    db 20, $00
    db 18, 14, 10, 30, 12

    dw $0800, 800, $0900
    db 20, $00
    db 14, 8, 10, 30, 6

    dw $0900, 900, $1000
    db 20, $00
    db 14, 8, 8, 17, 6

    dw $1000, 1000, $1100
    db 20, $00
    db 8, 8, 8, 17, 6

    dw $1100, 1100, $1200
    db 20, $00
    db 7, 7, 8, 15, 6

    dw $1200, 1200, $1299
    db 20, $00
    db 6, 6, 8, 15, 6

sTGM3SpeedCurveEnd::
    dw $FFFF

sTGM3SpeedCurveSpecialData::
    dw $FFFF    ; When do we get bones?
    dw 999      ; When do blocks turn invisible?
    dw 999      ; When is the hard kill screen?
    dw $0999    ; Same but in BCD.
    dw 998      ; If the last level lock isn't at x99, where is it?
    dw 3600     ; How long does the staff roll last after the kill screen, in frames.
    db $00      ; Big mode in staff roll?

sDEATSpeedCurve::
    dw $0000, 0, $0100
    db 20, $00
    db 18, 14, 12, 30, 12

    dw $0100, 100, $0200
    db 20, $00
    db 14, 8, 12, 26, 6

    dw $0200, 200, $0300
    db 20, $00
    db 14, 8, 11, 22, 6

    dw $0300, 300, $0400
    db 20, $00
    db 8, 8, 10, 18, 6

    dw $0400, 400, $0500
    db 20, $00
    db 7, 7, 8, 15, 5

    dw $0500, 500, $0600
    db 20, $00
    db 6, 6, 8, 15, 4

    dw $0600, 600, $0700
    db 20, $00
    db 6, 6, 8, 15, 4

    dw $0700, 700, $0800
    db 20, $00
    db 6, 6, 8, 15, 4

    dw $0800, 800, $0900
    db 20, $00
    db 6, 6, 8, 15, 4

    dw $0900, 900, $0999
    db 20, $00
    db 6, 6, 8, 15, 4

sDEATSpeedCurveEnd::
    dw $FFFF

sDEATSpeedCurveSpecialData::
    dw $FFFF   ; When do we get bones?
    dw 999     ; When do blocks turn invisible?
    dw 999     ; When is the hard kill screen?
    dw $0999   ; Same but in BCD.
    dw 998     ; If the last level lock isn't at x99, where is it?
    dw 3600    ; How long does the staff roll last after the kill screen, in frames.
    db $00     ; Big mode in staff roll?


sSHIRSpeedCurve::
    dw $0000, 0, $0100
    db 20, $00
    db 12, 8, 10, 18, 6

    dw $0100, 100, $0200
    db 20, $00
    db 12, 7, 8, 18, 5

    dw $0200, 200, $0300
    db 20, $00
    db 12, 6, 8, 17, 4

    dw $0300, 300, $0400
    db 20, $00
    db 6, 6, 8, 15, 4

    dw $0500, 500, $0600
    db 20, $00
    db 6, 5, 6, 13, 3

    dw $1100, 1100, $1200
    db 20, $00
    db 6, 5, 6, 10, 3

    dw $1200, 1200, $1300
    db 20, $00
    db 6, 5, 6, 8, 3

sSHIRSpeedCurveEnd::
    dw $FFFF

sSHIRSpeedCurveSpecialData::
    dw 1000    ; When do we get bones?
    dw $FFFF   ; When do blocks turn invisible?
    dw 1300    ; When is the hard kill screen?
    dw $1300   ; Same but in BCD.
    dw $FFFF   ; If the last level lock isn't at x99, where is it?
    dw 3600    ; How long does the staff roll last after the kill screen, in frames.
    db $FF     ; Big mode in staff roll?


sMYCOSpeedCurve::
    dw $0000, 0, $0100
    db 1, 4
    db 20, 20, 12, 30, 20

    dw $0005, 5, $0100
    db 1, 6
    db 20, 20, 12, 30, 20

    dw $0009, 9, $0100
    db 1, 8
    db 20, 20, 12, 30, 20

    dw $0013, 13, $0100
    db 1, 10
    db 20, 20, 12, 30, 20

    dw $0017, 17, $0100
    db 1, 12
    db 20, 20, 12, 30, 20

    dw $0021, 21, $0100
    db 1, 16
    db 20, 20, 12, 30, 20

    dw $0025, 25, $0100
    db 1, 32
    db 20, 20, 12, 30, 20

    dw $0029, 29, $0100
    db 1, 48
    db 20, 20, 12, 30, 20

    dw $0033, 33, $0100
    db 1, 64
    db 20, 20, 12, 30, 20

    dw $0037, 37, $0100
    db 1, 80
    db 20, 20, 12, 30, 20

    dw $0041, 41, $0100
    db 1, 96
    db 20, 20, 12, 30, 20

    dw $0045, 45, $0100
    db 1, 112
    db 20, 20, 12, 30, 20

    dw $0049, 49, $0100
    db 1, 128
    db 20, 20, 12, 30, 20

    dw $0053, 53, $0100
    db 1, 144
    db 20, 20, 12, 30, 20

    dw $0057, 57, $0100
    db 1, 160
    db 20, 20, 12, 30, 20

    dw $0061, 61, $0100
    db 1, 192
    db 20, 20, 12, 30, 20

    dw $0065, 65, $0100
    db 1, 224
    db 20, 20, 12, 30, 20

    dw $0069, 69, $0100
    db 1, $00
    db 20, 20, 12, 30, 20

    dw $0090, 90, $0100
    db 2, $00
    db 20, 20, 12, 30, 20

    dw $0100, 100, $0200
    db 3, $00
    db 20, 20, 12, 30, 20

    dw $0120, 120, $0200
    db 4, $00
    db 20, 20, 12, 30, 20

    dw $0135, 135, $0200
    db 5, $00
    db 20, 20, 12, 30, 20

    dw $0150, 150, $0200
    db 20, $00
    db 20, 20, 8, 30, 20

    dw $0200, 200, $0300
    db 20, $00
    db 20, 20, 8, 30, 20

    dw $0250, 250, $0300
    db 20, $00
    db 20, 20, 8, 30, 20

sMYCOSpeedCurveEnd::
    dw $FFFF

sMYCOSpeedCurveSpecialData::
    dw $FFFF    ; When do we get bones?
    dw $FFFF    ; When do blocks turn invisible?
    dw 300      ; When is the hard kill screen?
    dw $0300    ; Same but in BCD.
    dw $FFFF    ; If the last level lock isn't at x99, where is it?
    dw $FFFF    ; How long does the staff roll last after the kill screen, in frames.
    db $00      ; Big mode in staff roll?


sSharedTiles::
    DB $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $7C,$7C,$CE,$CE,$D6,$D6,$D6,$D6
    DB $D6,$D6,$E6,$E6,$7C,$7C,$00,$00
    DB $38,$38,$78,$78,$D8,$D8,$18,$18
    DB $18,$18,$18,$18,$18,$18,$00,$00
    DB $7C,$7C,$C6,$C6,$C6,$C6,$0C,$0C
    DB $38,$38,$E0,$E0,$FE,$FE,$00,$00
    DB $7C,$7C,$C6,$C6,$06,$06,$0C,$0C
    DB $06,$06,$C6,$C6,$7C,$7C,$00,$00
    DB $1C,$1C,$3C,$3C,$6C,$6C,$CC,$CC
    DB $FE,$FE,$0C,$0C,$0C,$0C,$00,$00
    DB $FE,$FE,$C0,$C0,$C0,$C0,$FC,$FC
    DB $06,$06,$06,$06,$FC,$FC,$00,$00
    DB $7C,$7C,$C6,$C6,$C0,$C0,$FC,$FC
    DB $C6,$C6,$C6,$C6,$7C,$7C,$00,$00
    DB $FE,$FE,$0C,$0C,$18,$18,$18,$18
    DB $30,$30,$30,$30,$30,$30,$00,$00
    DB $7C,$7C,$C6,$C6,$C6,$C6,$7C,$7C
    DB $C6,$C6,$C6,$C6,$7C,$7C,$00,$00
    DB $7C,$7C,$C6,$C6,$C6,$C6,$7E,$7E
    DB $06,$06,$C6,$C6,$7C,$7C,$00,$00
    DB $38,$38,$6C,$6C,$C6,$C6,$FE,$FE
    DB $C6,$C6,$C6,$C6,$C6,$C6,$00,$00
    DB $FC,$FC,$C6,$C6,$C6,$C6,$FC,$FC
    DB $C6,$C6,$C6,$C6,$FC,$FC,$00,$00
    DB $7C,$7C,$C6,$C6,$C0,$C0,$C0,$C0
    DB $C0,$C0,$C6,$C6,$7C,$7C,$00,$00
    DB $FC,$FC,$C6,$C6,$C6,$C6,$C6,$C6
    DB $C6,$C6,$C6,$C6,$FC,$FC,$00,$00
    DB $FE,$FE,$C0,$C0,$C0,$C0,$F8,$F8
    DB $C0,$C0,$C0,$C0,$FE,$FE,$00,$00
    DB $FE,$FE,$C0,$C0,$C0,$C0,$F8,$F8
    DB $C0,$C0,$C0,$C0,$C0,$C0,$00,$00
    DB $7C,$7C,$C6,$C6,$C0,$C0,$CE,$CE
    DB $C6,$C6,$C6,$C6,$7C,$7C,$00,$00
    DB $C6,$C6,$C6,$C6,$C6,$C6,$FE,$FE
    DB $C6,$C6,$C6,$C6,$C6,$C6,$00,$00
    DB $18,$18,$18,$18,$18,$18,$18,$18
    DB $18,$18,$18,$18,$18,$18,$00,$00
    DB $06,$06,$06,$06,$06,$06,$06,$06
    DB $66,$66,$66,$66,$3C,$3C,$00,$00
    DB $C6,$C6,$C6,$C6,$CC,$CC,$F8,$F8
    DB $CC,$CC,$C6,$C6,$C6,$C6,$00,$00
    DB $60,$60,$60,$60,$60,$60,$60,$60
    DB $60,$60,$60,$60,$7C,$7C,$00,$00
    DB $C6,$C6,$EE,$EE,$FE,$FE,$D6,$D6
    DB $C6,$C6,$C6,$C6,$C6,$C6,$00,$00
    DB $C6,$C6,$E6,$E6,$F6,$F6,$DE,$DE
    DB $CE,$CE,$C6,$C6,$C6,$C6,$00,$00
    DB $7C,$7C,$C6,$C6,$C6,$C6,$C6,$C6
    DB $C6,$C6,$C6,$C6,$7C,$7C,$00,$00
    DB $FC,$FC,$C6,$C6,$C6,$C6,$FC,$FC
    DB $C0,$C0,$C0,$C0,$C0,$C0,$00,$00
    DB $7C,$7C,$C6,$C6,$C6,$C6,$C6,$C6
    DB $D6,$D6,$DE,$DE,$7C,$7C,$00,$00
    DB $FC,$FC,$C6,$C6,$C6,$C6,$FC,$FC
    DB $F8,$F8,$DC,$DC,$CE,$CE,$00,$00
    DB $7E,$7E,$C0,$C0,$C0,$C0,$7C,$7C
    DB $06,$06,$06,$06,$FC,$FC,$00,$00
    DB $FC,$FC,$30,$30,$30,$30,$30,$30
    DB $30,$30,$30,$30,$30,$30,$00,$00
    DB $C6,$C6,$C6,$C6,$C6,$C6,$C6,$C6
    DB $C6,$C6,$C6,$C6,$7C,$7C,$00,$00
    DB $C6,$C6,$C6,$C6,$6C,$6C,$6C,$6C
    DB $6C,$6C,$38,$38,$38,$38,$00,$00
    DB $C6,$C6,$C6,$C6,$C6,$C6,$C6,$C6
    DB $D6,$D6,$D6,$D6,$6C,$6C,$00,$00
    DB $C6,$C6,$C6,$C6,$6C,$6C,$38,$38
    DB $6C,$6C,$C6,$C6,$C6,$C6,$00,$00
    DB $C6,$C6,$EE,$EE,$7C,$7C,$38,$38
    DB $70,$70,$E0,$E0,$C0,$C0,$00,$00
    DB $FE,$FE,$0C,$0C,$18,$18,$30,$30
    DB $60,$60,$C0,$C0,$FE,$FE,$00,$00
    DB $18,$18,$3C,$3C,$3C,$3C,$18,$18
    DB $18,$18,$00,$00,$18,$18,$00,$00
    DB $3C,$3C,$66,$66,$0C,$0C,$18,$18
    DB $18,$18,$00,$00,$18,$18,$00,$00
    DB $F0,$F0,$C0,$C0,$C0,$C0,$C0,$C0
    DB $C0,$C0,$C0,$C0,$F0,$F0,$00,$00
    DB $0F,$0F,$03,$03,$03,$03,$03,$03
    DB $03,$03,$03,$03,$0F,$0F,$00,$00
    DB $00,$00,$00,$00,$00,$00,$3C,$3C
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $18,$18,$18,$18,$08,$08,$10,$10
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $FF,$FF,$C3,$C3,$00,$00,$3C,$3C
    DB $00,$00,$C3,$C3,$FF,$FF,$00,$00
    DB $18,$18,$7E,$7E,$DB,$DB,$DB,$DB
    DB $DB,$DB,$7E,$7E,$18,$18,$00,$00
    DB $7E,$7E,$C3,$C3,$C3,$C3,$C3,$C3
    DB $66,$66,$A5,$A5,$E7,$E7,$00,$00
    DB $DB,$DB,$DB,$DB,$DB,$DB,$DB,$DB
    DB $7E,$7E,$18,$18,$18,$18,$00,$00
sSharedTilesEnd::

sTitleTiles::
    DB $80,$80,$80,$80,$00,$00,$00,$00
    DB $80,$80,$80,$80,$00,$00,$00,$00
    DB $80,$80,$80,$80,$03,$03,$0F,$0C
    DB $9F,$9F,$90,$9F,$10,$1F,$10,$1F
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $1E,$1E,$FE,$FE,$E0,$E0,$00,$00
    DB $1C,$1C,$3E,$22,$FF,$E3,$FF,$22
    DB $E7,$FB,$26,$FB,$3E,$E3,$1C,$FF
    DB $90,$9F,$90,$9F,$10,$1F,$10,$1F
    DB $9F,$9F,$80,$80,$00,$00,$00,$00
    DB $80,$80,$80,$80,$00,$00,$00,$00
    DB $80,$80,$80,$80,$00,$00,$00,$00
    DB $00,$FF,$00,$FF,$07,$FF,$07,$FC
    DB $F7,$FC,$13,$1E,$11,$1F,$10,$1F
    DB $10,$1F,$10,$1F,$10,$1F,$1F,$1F
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $1E,$1E,$1F,$1F,$01,$01,$00,$00
    DB $0E,$0E,$1F,$11,$FF,$F1,$FF,$11
    DB $F3,$FD,$13,$FD,$1F,$F1,$0E,$FF
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$C0,$C0,$C0,$C0,$00,$00
    DB $00,$00,$00,$00,$FF,$FF,$FF,$0D
    DB $FF,$F1,$1F,$F1,$1F,$F1,$1F,$F1
    DB $00,$FF,$00,$FF,$E0,$FF,$E0,$3F
    DB $FF,$3F,$DF,$71,$9F,$F1,$1F,$F1
    DB $1F,$F1,$1F,$F3,$1C,$FC,$F0,$F0
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $1F,$F1,$1F,$F1,$1F,$F3,$1C,$FC
    DB $F0,$F0,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $18,$18,$18,$18,$18,$18,$18,$18
    DB $18,$18,$18,$18,$18,$18,$18,$18
    DB $10,$10,$20,$20,$41,$41,$41,$41
    DB $47,$47,$57,$57,$30,$30,$70,$70
    DB $0F,$0F,$E8,$EF,$08,$0F,$AF,$AF
    DB $00,$00,$B6,$B6,$00,$00,$AE,$AE
    DB $0E,$0E,$0C,$0C,$EA,$EA,$E2,$E2
    DB $82,$82,$82,$82,$04,$04,$08,$08
    DB $C3,$03,$CE,$06,$CC,$00,$CD,$01
    DB $CD,$01,$CF,$00,$CF,$01,$C1,$01
    DB $09,$01,$85,$01,$61,$01,$17,$07
    DB $03,$03,$01,$01,$00,$00,$0F,$0F
    DB $FF,$C0,$7F,$60,$CC,$C0,$8C,$80
    DB $E0,$80,$60,$00,$FE,$80,$FE,$80
    DB $90,$80,$A1,$80,$86,$80,$E8,$E0
    DB $C0,$C0,$80,$80,$00,$00,$F0,$F0
    DB $FF,$FF,$01,$FF,$01,$FF,$9F,$FF
    DB $90,$F0,$90,$F0,$90,$F0,$F0,$F0
    DB $05,$00,$00,$00,$05,$00,$05,$00
    DB $60,$60,$E0,$E2,$F0,$F7,$FF,$FF
    DB $00,$00,$00,$00,$FF,$FF,$FF,$FF
    DB $18,$18,$18,$18,$18,$18,$18,$18
    DB $62,$62,$15,$15,$15,$15,$25,$25
    DB $45,$45,$72,$72,$00,$00,$FF,$FF
    DB $1F,$1F,$18,$1F,$18,$1F,$18,$1F
    DB $1F,$1F,$18,$18,$18,$18,$18,$18
    DB $3C,$3C,$24,$3C,$E7,$FF,$81,$FF
    DB $81,$FF,$E7,$FF,$24,$3C,$3C,$3C
    DB $98,$98,$68,$F8,$08,$F8,$08,$F8
    DB $98,$F8,$60,$60,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$DB,$DB,$DB,$DB,$00,$00
    DB $F8,$F8,$94,$94,$B2,$92,$9E,$9E
    DB $BA,$82,$82,$82,$FE,$FE,$00,$00
    DB $30,$30,$30,$30,$F0,$F0,$F0,$F0
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $30,$30,$30,$30,$30,$30,$30,$30
    DB $30,$30,$30,$30,$30,$30,$30,$30
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$C0,$C0,$C0,$C0,$00,$00
    DB $70,$70,$D8,$D8,$D8,$D8,$D8,$D8
    DB $D8,$D8,$DB,$DB,$73,$73,$00,$00
    DB $03,$03,$0F,$0F,$3F,$3F,$FF,$FF
    DB $3F,$3F,$0F,$0F,$03,$03,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$CC,$CC,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $19,$19,$3F,$3F,$66,$66,$66,$66
    DB $FF,$FF,$FF,$FF,$79,$79,$30,$30
    DB $00,$00,$00,$00,$01,$00,$00,$00
    DB $00,$00,$01,$00,$00,$00,$00,$00
    DB $80,$80,$C0,$C0,$60,$60,$60,$60
    DB $F0,$F0,$F0,$F0,$E0,$E0,$C0,$C0
    DB $00,$00,$87,$00,$85,$00,$87,$00 ; Version
    DB $85,$00,$D7,$00,$00,$00,$00,$00 ; Version
    DB $C0,$C0,$F0,$F0,$FC,$FC,$FF,$FF
    DB $FC,$FC,$F0,$F0,$C0,$C0,$00,$00
    DB $C0,$00,$F0,$00,$FC,$00,$FF,$00
    DB $FC,$00,$F0,$00,$C0,$00,$00,$00
    DB $00,$00,$00,$00,$FF,$FF,$FF,$FF
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $18,$18,$18,$18,$FF,$FF,$FF,$FF
    DB $18,$18,$18,$18,$18,$18,$18,$18
    DB $00,$75,$00,$47,$00,$55,$00,$55
    DB $00,$75,$00,$00,$00,$3E,$00,$00
    DB $00,$75,$00,$47,$00,$55,$00,$55
    DB $00,$75,$00,$00,$00,$36,$00,$00
    DB $00,$75,$00,$47,$00,$55,$00,$55
    DB $00,$75,$00,$00,$00,$2A,$00,$00
    DB $00,$00,$00,$3E,$00,$7F,$00,$6B
    DB $00,$3E,$00,$2A,$00,$00,$00,$00
    DB $00,$00,$00,$FF,$20,$81,$76,$81
    DB $20,$81,$00,$FF,$00,$00,$00,$00
    DB $00,$26,$00,$55,$00,$76,$00,$55
    DB $00,$55,$00,$00,$00,$3E,$00,$00
    DB $00,$26,$00,$55,$00,$76,$00,$55
    DB $00,$55,$00,$00,$00,$36,$00,$00
    DB $00,$00,$00,$FF,$20,$81,$76,$81
    DB $20,$81,$00,$FF,$00,$00,$00,$00
    DB $18,$18,$2C,$34,$62,$5E,$76,$6E
    DB $24,$3C,$24,$3C,$18,$18,$00,$00
    DB $02,$70,$02,$40,$02,$60,$07,$40
    DB $07,$40,$02,$00,$00,$1F,$00,$00
    DB $02,$70,$02,$40,$02,$70,$07,$10
    DB $07,$70,$02,$00,$00,$1F,$00,$00
    DB $02,$50,$02,$50,$02,$70,$07,$50
    DB $07,$50,$02,$00,$00,$1F,$00,$00
    DB $02,$40,$02,$40,$02,$40,$07,$40
    DB $07,$70,$02,$00,$00,$1F,$00,$00
    DB $02,$50,$02,$50,$02,$20,$07,$50
    DB $07,$50,$02,$00,$00,$1F,$00,$00
    DB $00,$3E,$02,$61,$04,$51,$24,$49
    DB $00,$45,$30,$43,$00,$3E,$00,$00
    DB $00,$00,$00,$62,$00,$15,$00,$25
    DB $00,$45,$00,$72,$00,$00,$00,$00
    DB $02,$02,$05,$05,$05,$05,$03,$03
    DB $01,$01,$05,$05,$02,$02,$00,$00
    DB $02,$02,$05,$05,$05,$05,$02,$02
    DB $05,$05,$05,$05,$02,$02,$00,$00
    DB $07,$07,$01,$01,$01,$01,$02,$02
    DB $02,$02,$02,$02,$02,$02,$00,$00
    DB $02,$02,$05,$05,$04,$04,$06,$06
    DB $05,$05,$05,$05,$02,$02,$00,$00
    DB $07,$07,$04,$04,$04,$04,$06,$06
    DB $01,$01,$01,$01,$06,$06,$00,$00
    DB $05,$05,$05,$05,$05,$05,$07,$07
    DB $01,$01,$01,$01,$01,$01,$00,$00
    DB $06,$06,$01,$01,$01,$01,$02,$02
    DB $01,$01,$01,$01,$06,$06,$00,$00
    DB $06,$06,$01,$01,$01,$01,$02,$02
    DB $04,$04,$04,$04,$07,$07,$00,$00
    DB $02,$02,$06,$06,$02,$02,$02,$02
    DB $02,$02,$02,$02,$07,$07,$00,$00
    DB $E2,$E2,$86,$86,$82,$82,$E2,$E2
    DB $22,$22,$22,$22,$E7,$E7,$00,$00
    DB $E6,$E6,$81,$81,$81,$81,$E2,$E2
    DB $24,$24,$24,$24,$E7,$E7,$00,$00
    DB $E6,$E6,$81,$81,$81,$81,$E2,$E2
    DB $21,$21,$21,$21,$E6,$E6,$00,$00
    DB $E5,$E5,$85,$85,$85,$85,$E7,$E7
    DB $21,$21,$21,$21,$E1,$E1,$00,$00
    DB $E7,$E7,$84,$84,$84,$84,$E6,$E6
    DB $21,$21,$21,$21,$E6,$E6,$00,$00
    DB $E2,$E2,$85,$85,$84,$84,$E6,$E6
    DB $25,$25,$25,$25,$E2,$E2,$00,$00
    DB $E7,$E7,$81,$81,$81,$81,$E2,$E2
    DB $22,$22,$22,$22,$E2,$E2,$00,$00
    DB $E2,$E2,$85,$85,$85,$85,$E2,$E2
    DB $25,$25,$25,$25,$E2,$E2,$00,$00
    DB $E2,$E2,$85,$85,$85,$85,$E3,$E3
    DB $21,$21,$25,$25,$E2,$E2,$00,$00
    DB $D2,$D2,$95,$95,$95,$95,$D5,$D5
    DB $55,$55,$55,$55,$D2,$D2,$00,$00
    DB $D2,$D2,$96,$96,$92,$92,$D2,$D2
    DB $52,$52,$52,$52,$D7,$D7,$00,$00
    DB $D6,$D6,$91,$91,$91,$91,$D2,$D2
    DB $54,$54,$54,$54,$D7,$D7,$00,$00
    DB $D6,$D6,$91,$91,$91,$91,$D2,$D2
    DB $51,$51,$51,$51,$D6,$D6,$00,$00
    DB $02,$02,$06,$06,$02,$02,$A2,$A2
    DB $E2,$E2,$A2,$A2,$A7,$A7,$00,$00
    DB $06,$06,$01,$01,$01,$01,$A2,$A2
    DB $E4,$E4,$A4,$A4,$A7,$A7,$00,$00
    DB $06,$06,$01,$01,$01,$01,$A2,$A2
    DB $E1,$E1,$A1,$A1,$A6,$A6,$00,$00
    DB $05,$05,$05,$05,$05,$05,$A7,$A7
    DB $E1,$E1,$A1,$A1,$A1,$A1,$00,$00
    DB $07,$07,$04,$04,$04,$04,$A6,$A6
    DB $E1,$E1,$A1,$A1,$A6,$A6,$00,$00
    DB $02,$02,$05,$05,$04,$04,$A6,$A6
    DB $E5,$E5,$A5,$A5,$A2,$A2,$00,$00
    DB $07,$07,$01,$01,$01,$01,$A2,$A2
    DB $E2,$E2,$A2,$A2,$A2,$A2,$00,$00
    DB $02,$02,$05,$05,$05,$05,$A2,$A2
    DB $E5,$E5,$A5,$A5,$A2,$A2,$00,$00
    DB $02,$02,$05,$05,$05,$05,$A3,$A3
    DB $E1,$E1,$A5,$A5,$A2,$A2,$00,$00
    DB $A0,$A0,$E0,$E0,$A0,$A0,$A0,$A0
    DB $A0,$A0,$A0,$A0,$A0,$A0,$00,$00
    DB $AA,$AA,$EA,$EA,$AE,$AE,$AE,$AE
    DB $A4,$A4,$A4,$A4,$A4,$A4,$00,$00
    DB $A4,$A4,$E4,$E4,$AE,$AE,$AA,$AA
    DB $AE,$AE,$A4,$A4,$A4,$A4,$00,$00
    DB $AE,$AE,$E0,$E0,$A0,$A0,$A4,$A4
    DB $A0,$A0,$A0,$A0,$AE,$AE,$00,$00
    DB $A4,$A4,$EA,$EA,$AA,$AA,$AA,$AA
    DB $A4,$A4,$AA,$AA,$AA,$AA,$00,$00
    DB $EA,$EA,$8E,$8E,$8A,$8A,$AA,$AA
    DB $AA,$AA,$AA,$AA,$EA,$EA,$00,$00
    DB $00,$00,$66,$66,$66,$66,$3C,$3C
    DB $3C,$3C,$66,$66,$66,$66,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $3C,$3C,$42,$42,$81,$81,$81,$81
    DB $81,$81,$81,$81,$42,$42,$3C,$3C
    DB $3C,$3C,$4E,$4E,$8D,$8D,$89,$89
    DB $81,$81,$81,$81,$42,$42,$3C,$3C
    DB $3C,$3C,$4E,$4E,$8F,$8F,$8F,$8F
    DB $81,$81,$81,$81,$42,$42,$3C,$3C
    DB $3C,$3C,$4E,$4E,$8F,$8F,$8F,$8F
    DB $8F,$8F,$87,$87,$42,$42,$3C,$3C
    DB $3C,$3C,$4E,$4E,$8F,$8F,$8F,$8F
    DB $8F,$8F,$8F,$8F,$4E,$4E,$3C,$3C
    DB $3C,$3C,$4E,$4E,$8F,$8F,$8F,$8F
    DB $9F,$9F,$BF,$BF,$7E,$7E,$3C,$3C
    DB $3C,$3C,$4E,$4E,$8F,$8F,$8F,$8F
    DB $FF,$FF,$FF,$FF,$7E,$7E,$3C,$3C
    DB $3C,$3C,$4E,$4E,$EF,$EF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$7E,$7E,$3C,$3C
    DB $3C,$3C,$7E,$7E,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$7E,$7E,$3C,$3C
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$30,$00,$78,$00
    DB $78,$00,$30,$00,$07,$00,$0B,$04
    DB $35,$0A,$6B,$14,$70,$0F,$30,$0F
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$02,$00,$00,$20,$00
    DB $50,$00,$2A,$06,$1E,$01,$BF,$00
    DB $EE,$11,$95,$6A,$02,$FD,$B0,$FF
    DB $19,$07,$0C,$03,$06,$01,$CD,$03
    DB $E8,$07,$E0,$07,$CC,$03,$0F,$00
    DB $16,$00,$28,$00,$10,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $7E,$FF,$FF,$FF,$BF,$FF,$47,$FF
    DB $00,$FF,$12,$ED,$AF,$50,$6F,$00
    DB $06,$00,$00,$00,$04,$00,$0A,$00
    DB $04,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $04,$00,$0A,$00,$14,$00,$20,$60
    DB $F0,$11,$61,$90,$61,$80,$38,$C0
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$20,$00,$00,$20,$A8,$70
    DB $00,$20,$20,$00,$00,$00,$60,$E0
    DB $E0,$10,$F0,$00,$F0,$00,$E1,$00
    DB $DE,$E0,$AB,$F4,$45,$FA,$E2,$FD
    DB $C8,$FF,$05,$FF,$30,$CF,$F9,$06
    DB $D5,$0A,$1B,$04,$0E,$00,$00,$20
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $02,$00,$A1,$10,$35,$CA,$47,$F8
    DB $A2,$FD,$01,$FE,$E7,$18,$CD,$00
    DB $9C,$00,$38,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$7F,$00,$7F,$00
    DB $7F,$00,$7F,$00,$7F,$00,$7E,$00
    DB $7E,$00,$7E,$00,$7E,$00,$7E,$00
    DB $00,$7E,$7F,$00,$00,$7F,$7F,$00
    DB $00,$00,$00,$00,$F8,$00,$FC,$00
    DB $FE,$00,$FF,$00,$FF,$00,$3F,$00
    DB $3F,$00,$3F,$00,$3F,$00,$3F,$00
    DB $00,$3F,$FF,$00,$00,$FE,$FC,$00
    DB $80,$7F,$80,$7F,$80,$7E,$81,$7E
    DB $FF,$7E,$81,$7E,$FF,$7E,$80,$7E
    DB $7F,$7F,$7F,$7F,$7F,$7F,$7F,$7F
    DB $7F,$7F,$00,$00,$00,$00,$00,$00
    DB $01,$FE,$00,$FF,$C0,$3F,$C0,$3F
    DB $FF,$3F,$C0,$3F,$FF,$3F,$C0,$3F
    DB $FF,$FF,$FF,$FF,$FE,$FE,$FC,$FC
    DB $F8,$F8,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$7E,$00,$7E,$00
    DB $7E,$00,$7E,$00,$7E,$00,$7E,$00
    DB $7E,$00,$7E,$00,$7E,$00,$7E,$00
    DB $00,$7E,$7E,$00,$00,$7E,$7E,$00
    DB $00,$00,$00,$00,$00,$20,$00,$00
    DB $00,$00,$04,$00,$0A,$00,$04,$00
    DB $60,$00,$F0,$00,$F0,$00,$60,$00
    DB $02,$00,$1D,$00,$3E,$00,$F6,$08
    DB $81,$7E,$81,$7E,$81,$7E,$81,$7E
    DB $7F,$7E,$01,$7E,$7E,$7E,$00,$7E
    DB $7F,$7F,$7F,$7F,$3F,$3F,$1F,$1F
    DB $0F,$0F,$00,$00,$00,$00,$00,$00
    DB $EB,$14,$97,$68,$29,$F6,$50,$FF
    DB $94,$6F,$F9,$06,$FF,$00,$03,$00
    DB $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$0F,$00,$1F,$00
    DB $3F,$00,$7F,$00,$7F,$00,$7E,$00
    DB $7E,$00,$7E,$00,$7E,$00,$7E,$00
    DB $00,$7E,$7E,$00,$00,$7E,$7E,$00
    DB $00,$00,$00,$00,$F8,$00,$FC,$00
    DB $FE,$00,$FF,$00,$FF,$00,$3F,$00
    DB $3F,$00,$3F,$00,$3F,$00,$3F,$00
    DB $00,$3F,$3F,$00,$00,$3F,$3F,$00
    DB $00,$7E,$00,$7E,$01,$7E,$81,$7E
    DB $FF,$7E,$81,$7E,$FF,$7E,$80,$7E
    DB $7F,$7F,$7F,$7F,$3F,$3F,$1F,$1F
    DB $0F,$0F,$00,$00,$00,$00,$00,$00
    DB $00,$3F,$00,$3F,$C0,$3F,$C0,$3F
    DB $FF,$3F,$C0,$3F,$FF,$3F,$00,$3F
    DB $FF,$FF,$FF,$FF,$FE,$FE,$FC,$FC
    DB $F8,$F8,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$0F,$00,$1F,$00
    DB $3F,$00,$7F,$00,$7F,$00,$7E,$00
    DB $7E,$00,$7E,$00,$7E,$00,$7E,$00
    DB $00,$7E,$7E,$00,$00,$7E,$7E,$00
    DB $00,$00,$00,$00,$FF,$00,$FF,$00
    DB $FF,$00,$FF,$00,$FF,$00,$00,$00
    DB $00,$00,$0C,$00,$1E,$00,$1E,$00
    DB $0C,$00,$20,$00,$50,$00,$2E,$00
    DB $00,$7E,$01,$7E,$81,$7E,$81,$7E
    DB $FF,$7E,$01,$7E,$7E,$7E,$00,$7E
    DB $7F,$7F,$7F,$7F,$3F,$3F,$1F,$1F
    DB $0F,$0F,$00,$00,$00,$00,$00,$00
    DB $1B,$04,$D7,$08,$F9,$06,$90,$6F
    DB $2A,$FF,$59,$F6,$9E,$61,$73,$00
    DB $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$7E,$00,$7E,$00
    DB $7E,$00,$7E,$00,$7E,$00,$7E,$00
    DB $7E,$00,$7E,$00,$7E,$00,$7E,$00
    DB $00,$7E,$7F,$00,$00,$7F,$7F,$00
    DB $00,$00,$00,$00,$3F,$00,$3F,$00
    DB $3F,$00,$3F,$00,$3F,$00,$3F,$00
    DB $3F,$00,$3F,$00,$3F,$00,$3F,$00
    DB $00,$3F,$FF,$00,$00,$FE,$FC,$00
    DB $00,$7F,$00,$7F,$00,$7E,$81,$7E
    DB $FF,$7E,$81,$7E,$FF,$7E,$80,$7E
    DB $7F,$7E,$7E,$7E,$7E,$7E,$7E,$7E
    DB $7E,$7E,$00,$00,$00,$00,$00,$00
    DB $01,$FE,$00,$FF,$C0,$3F,$C0,$3F
    DB $FF,$3F,$C0,$3F,$FF,$3F,$C0,$3F
    DB $7F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
    DB $3F,$3F,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$0F,$00,$1F,$00
    DB $3F,$00,$7F,$00,$7F,$00,$7E,$00
    DB $7E,$00,$7E,$00,$7E,$00,$7E,$00
    DB $00,$7E,$7F,$00,$00,$7F,$3F,$00
    DB $00,$00,$00,$00,$FF,$00,$FF,$00
    DB $FF,$00,$FF,$00,$FF,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$F8,$00,$00,$FC,$FE,$00
    DB $40,$1F,$80,$0F,$1D,$00,$2F,$00
    DB $F7,$00,$5E,$00,$0D,$00,$00,$00
    DB $7F,$7F,$7F,$7F,$7F,$7F,$7F,$7F
    DB $7F,$7F,$00,$00,$00,$00,$00,$00
    DB $00,$FF,$00,$FF,$C0,$3F,$C0,$3F
    DB $FF,$3F,$C0,$3F,$FF,$3F,$C0,$3F
    DB $FF,$FF,$FF,$FF,$FE,$FE,$FC,$FC
    DB $F8,$F8,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$10,$00,$00
    DB $00,$00,$00,$00,$18,$00,$3C,$00
    DB $3C,$00,$18,$00,$01,$00,$03,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$0A,$00,$14,$00
    DB $08,$00,$00,$00,$00,$00,$00,$00
    DB $1D,$00,$3F,$00,$E8,$17,$90,$6F
    DB $1F,$00,$36,$09,$E8,$17,$F5,$0B
    DB $BC,$43,$E6,$01,$C3,$24,$43,$C2
    DB $01,$01,$10,$00,$00,$10,$54,$38
    DB $00,$10,$10,$00,$00,$00,$00,$00
    DB $49,$B7,$83,$7F,$8F,$FF,$5B,$FF
    DB $A5,$FF,$42,$BF,$E0,$1F,$B4,$4B
    DB $3B,$A4,$14,$18,$10,$00,$28,$00
    DB $10,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$0C,$00
    DB $1E,$00,$1E,$00,$0D,$00,$E3,$00
    DB $F2,$01,$FF,$00,$E7,$18,$42,$BD
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$10,$00,$28,$00
    DB $17,$00,$0F,$00,$CF,$00,$7F,$80
    DB $AC,$53,$10,$EF,$02,$FF,$04,$FF
    DB $11,$FF,$BB,$FF,$FF,$FF,$FF,$FF
    DB $FD,$FF,$6A,$FF,$14,$FF,$01,$FE
    DB $43,$BC,$A7,$58,$DE,$20,$71,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $AA,$FF,$D4,$FF,$E0,$FF,$E4,$FF
    DB $CA,$FF,$04,$FF,$C8,$37,$7C,$03
    DB $07,$00,$33,$00,$F9,$00,$78,$00
    DB $B0,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$07,$07,$06,$06,$00,$06
    DB $07,$07,$00,$06,$00,$06,$00,$06
    DB $00,$00,$9E,$9E,$59,$59,$00,$59
    DB $5E,$5E,$00,$59,$00,$59,$00,$99
    DB $00,$00,$8E,$8E,$59,$59,$00,$59
    DB $9F,$9F,$00,$19,$00,$19,$00,$19
    DB $00,$00,$39,$39,$65,$65,$00,$64
    DB $7C,$7C,$00,$64,$00,$64,$00,$64
    DB $00,$00,$18,$1C,$2A,$3E,$51,$7E
    DB $23,$7C,$57,$28,$2E,$10,$1C,$00
    DB $E0,$60,$A8,$F8,$40,$FC,$82,$FC
    DB $46,$F8,$AF,$70,$FD,$06,$EA,$3F
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$20,$00,$00,$00,$00
    DB $FF,$00,$FF,$18,$EB,$3C,$56,$79
    DB $7F,$01,$3E,$03,$14,$1F,$02,$07
    DB $00,$00,$75,$75,$75,$75,$00,$75
    DB $6D,$6D,$00,$6D,$00,$6D,$00,$6D
    DB $00,$00,$38,$38,$64,$64,$00,$60
    DB $38,$38,$00,$04,$00,$64,$00,$38
    DB $00,$00,$E3,$E3,$96,$96,$00,$96
    DB $96,$96,$00,$96,$00,$96,$00,$E3
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$40,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$10,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $51,$FE,$A7,$F8,$4B,$F4,$96,$E8
    DB $2E,$D0,$1C,$E0,$78,$80,$E0,$00
    DB $07,$07,$1D,$1E,$31,$3F,$63,$7F
    DB $47,$7E,$8F,$FC,$07,$F8,$3F,$C0
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
sTitleTilesEnd::

sGameplayTilesM::
    DB $00,$FF,$7E,$99,$66,$BD,$66,$BD
    DB $66,$BD,$7E,$99,$00,$FF,$00,$00
    DB $00,$FF,$36,$C9,$6C,$93,$5A,$A5
    DB $36,$C9,$6C,$93,$00,$FF,$00,$00
    DB $00,$FF,$42,$BD,$7E,$81,$7E,$81
    DB $7E,$81,$42,$BD,$00,$FF,$00,$00
    DB $00,$FF,$3C,$C3,$5A,$A5,$7E,$81
    DB $5A,$A5,$3C,$C3,$00,$FF,$00,$00
    DB $00,$FF,$5A,$A5,$42,$BD,$42,$BD
    DB $42,$BD,$5A,$A5,$00,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$7E,$81,$00,$FF,$00,$00
    DB $00,$FF,$54,$AB,$2A,$D5,$54,$AB
    DB $2A,$D5,$54,$AB,$00,$FF,$00,$00
    DB $FF,$FF,$FF,$99,$E7,$BD,$E7,$BD
    DB $E7,$BD,$FF,$99,$FF,$FF,$00,$00
    DB $FF,$FF,$B7,$C9,$ED,$93,$DB,$A5
    DB $B7,$C9,$ED,$93,$FF,$FF,$00,$00
    DB $FF,$FF,$C3,$BD,$FF,$81,$FF,$81
    DB $FF,$81,$C3,$BD,$FF,$FF,$00,$00
    DB $FF,$FF,$BD,$C3,$DB,$A5,$FF,$81
    DB $DB,$A5,$BD,$C3,$FF,$FF,$00,$00
    DB $FF,$FF,$DB,$A5,$C3,$BD,$C3,$BD
    DB $C3,$BD,$DB,$A5,$FF,$FF,$00,$00
    DB $FF,$FF,$FF,$81,$FF,$81,$FF,$81
    DB $FF,$81,$FF,$81,$FF,$FF,$00,$00
    DB $FF,$FF,$D5,$AB,$AB,$D5,$D5,$AB
    DB $AB,$D5,$D5,$AB,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$99,$66,$BD,$66,$BD
    DB $66,$BD,$FE,$BB,$FF,$FF,$00,$00
    DB $00,$FF,$36,$C9,$6C,$93,$5A,$A5
    DB $36,$C9,$EE,$BB,$FF,$FF,$00,$00
    DB $00,$FF,$42,$BD,$7E,$81,$7E,$81
    DB $7E,$81,$EA,$BF,$FF,$FF,$00,$00
    DB $00,$FF,$3C,$C3,$5A,$A5,$7E,$81
    DB $5A,$A5,$BE,$EB,$FF,$FF,$00,$00
    DB $00,$FF,$5A,$A5,$42,$BD,$42,$BD
    DB $42,$BD,$FA,$AF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$FE,$AB,$FF,$FF,$00,$00
    DB $00,$FF,$54,$AB,$2A,$D5,$54,$AB
    DB $2A,$D5,$FE,$AB,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$99,$66,$BD,$66,$BD
    DB $EE,$BF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$36,$C9,$6C,$93,$5A,$A5
    DB $BE,$EB,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$42,$BD,$7E,$81,$7E,$81
    DB $FE,$AB,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$3C,$C3,$5A,$A5,$7E,$81
    DB $FA,$AF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$5A,$A5,$42,$BD,$42,$BD
    DB $EA,$BF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $FE,$AB,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$54,$AB,$2A,$D5,$54,$AB
    DB $AA,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$99,$66,$BD,$EE,$BF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$36,$C9,$6C,$93,$FA,$AF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$42,$BD,$7E,$81,$FE,$AB
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$3C,$C3,$5A,$A5,$FE,$AB
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$5A,$A5,$42,$BD,$EA,$BF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$FE,$AB
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$54,$AB,$2A,$D5,$FE,$AB
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$99,$EE,$BF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$36,$C9,$EE,$BB,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$42,$BD,$FE,$AB,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$3C,$C3,$FA,$AF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$5A,$A5,$EA,$BF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$FE,$AB,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$54,$AB,$AA,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$FE,$BB,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$BE,$EB,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$EA,$BF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$BE,$EB,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$FA,$AF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$FE,$AB,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$FE,$AB,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $AA,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $AA,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $AA,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $AA,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $AA,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $AA,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $AA,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $FF,$FF,$FF,$99,$FF,$BD,$FF,$BD
    DB $FF,$BD,$FF,$99,$FF,$FF,$00,$00
    DB $00,$FF,$66,$99,$42,$BD,$42,$BD
    DB $42,$BD,$66,$99,$00,$FF,$00,$00
    DB $FF,$FF,$AA,$FF,$00,$FF,$AA,$55
    DB $00,$FF,$55,$FF,$FF,$FF,$00,$00
    DB $DB,$E7,$A5,$C3,$42,$81,$91,$00
    DB $42,$81,$A5,$C3,$DB,$E7,$00,$00
    DB $00,$00,$00,$00,$00,$00,$10,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $E6,$F8,$E6,$F8,$E6,$F8,$E6,$F8
    DB $E6,$F8,$E6,$F8,$E6,$F8,$00,$00
    DB $E6,$F8,$E6,$F8,$FF,$FF,$E7,$F8
    DB $FF,$FF,$E6,$F8,$E6,$F8,$00,$00
    DB $1C,$14,$1C,$14,$1C,$14,$1C,$14
    DB $1C,$14,$1C,$14,$1C,$14,$00,$00
    DB $00,$00,$00,$00,$FF,$FF,$FF,$00
    DB $FF,$FF,$00,$00,$00,$00,$00,$00
    DB $1C,$14,$1C,$14,$FC,$E4,$F8,$08
    DB $FC,$E4,$1C,$14,$1C,$14,$00,$00
    DB $00,$00,$00,$00,$00,$FF,$FF,$FF
    DB $00,$FF,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$FF,$FF,$00,$FF
    DB $FF,$FF,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$FF,$FF,$D5,$AA
    DB $AA,$D5,$FF,$FF,$00,$00,$00,$00
    DB $00,$00,$00,$00,$FF,$FF,$55,$AB
    DB $AB,$55,$FF,$FF,$00,$00,$00,$00
    DB $3F,$3F,$35,$2A,$2A,$35,$3F,$3E
    DB $02,$03,$03,$02,$02,$03,$03,$03
    DB $C0,$C0,$40,$C0,$C0,$40,$40,$C0
    DB $FC,$7C,$54,$AC,$AC,$54,$FC,$FC
    DB $03,$03,$03,$02,$02,$03,$03,$02
    DB $3E,$3F,$35,$2A,$2A,$35,$3F,$3F
    DB $FC,$FC,$54,$AC,$AC,$54,$7C,$FC
    DB $C0,$40,$40,$C0,$C0,$40,$C0,$C0
    DB $3F,$3F,$35,$2A,$2A,$35,$3F,$3F
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $FC,$FC,$54,$AC,$AC,$54,$F4,$EC
    DB $2C,$34,$34,$2C,$2C,$34,$3C,$3C
    DB $3F,$3F,$35,$2A,$2A,$35,$37,$2F
    DB $2C,$34,$34,$2C,$2C,$34,$3C,$3C
    DB $FC,$FC,$54,$AC,$AC,$54,$FC,$FC
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $0F,$0F,$0A,$0D,$0D,$0A,$0A,$0D
    DB $0D,$0A,$0A,$0D,$0D,$0A,$0F,$0F
    DB $F0,$F0,$B0,$50,$50,$B0,$B0,$50
    DB $50,$B0,$B0,$50,$50,$B0,$F0,$F0
    DB $3F,$3F,$35,$2A,$2A,$35,$3F,$3E
    DB $02,$03,$03,$02,$02,$03,$03,$03
    DB $FC,$FC,$54,$AC,$AC,$54,$7C,$FC
    DB $C0,$40,$40,$C0,$C0,$40,$C0,$C0
    DB $00,$75,$00,$47,$00,$55,$00,$55
    DB $00,$75,$00,$00,$00,$3E,$00,$00
    DB $00,$75,$00,$47,$00,$55,$00,$55
    DB $00,$75,$00,$00,$00,$36,$00,$00
    DB $00,$75,$00,$47,$00,$55,$00,$55
    DB $00,$75,$00,$00,$00,$2A,$00,$00
    DB $00,$00,$00,$3E,$00,$7F,$00,$6B
    DB $00,$3E,$00,$2A,$00,$00,$00,$00
    DB $00,$00,$00,$FF,$20,$81,$76,$81
    DB $20,$81,$00,$FF,$00,$00,$00,$00
    DB $00,$26,$00,$55,$00,$76,$00,$55
    DB $00,$55,$00,$00,$00,$3E,$00,$00
    DB $00,$26,$00,$55,$00,$76,$00,$55
    DB $00,$55,$00,$00,$00,$36,$00,$00
    DB $00,$00,$00,$FF,$20,$81,$76,$81
    DB $20,$81,$00,$FF,$00,$00,$00,$00
    DB $02,$70,$02,$40,$02,$60,$07,$40
    DB $07,$40,$02,$00,$00,$1F,$00,$00
    DB $02,$70,$02,$40,$02,$70,$07,$10
    DB $07,$70,$02,$00,$00,$1F,$00,$00
    DB $02,$50,$02,$50,$02,$70,$07,$50
    DB $07,$50,$02,$00,$00,$1F,$00,$00
    DB $02,$40,$02,$40,$02,$40,$07,$40
    DB $07,$70,$02,$00,$00,$1F,$00,$00
    DB $02,$50,$02,$50,$02,$20,$07,$50
    DB $07,$50,$02,$00,$00,$1F,$00,$00
    DB $00,$3E,$02,$61,$04,$51,$24,$49
    DB $00,$45,$30,$43,$00,$3E,$00,$00
    DB $00,$00,$00,$62,$00,$15,$00,$25
    DB $00,$45,$00,$72,$00,$00,$00,$00
    DB $00,$FF,$00,$FF,$00,$FF,$00,$FF
    DB $00,$FF,$00,$FF,$00,$FF,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $7C,$FF,$C6,$FF,$C0,$FF,$CE,$FF
    DB $C6,$FF,$C6,$FF,$7C,$FF,$00,$00
    DB $38,$FF,$6C,$FF,$C6,$FF,$FE,$FF
    DB $C6,$FF,$C6,$FF,$C6,$FF,$00,$00
    DB $C6,$FF,$EE,$FF,$FE,$FF,$D6,$FF
    DB $C6,$FF,$C6,$FF,$C6,$FF,$00,$00
    DB $FE,$FF,$C0,$FF,$C0,$FF,$F8,$FF
    DB $C0,$FF,$C0,$FF,$FE,$FF,$00,$00
    DB $00,$FF,$00,$FF,$00,$FF,$00,$FF
    DB $00,$FF,$00,$FF,$00,$FF,$00,$00
    DB $7C,$FF,$C6,$FF,$C6,$FF,$C6,$FF
    DB $C6,$FF,$C6,$FF,$7C,$FF,$00,$00
    DB $C6,$FF,$C6,$FF,$6C,$FF,$6C,$FF
    DB $6C,$FF,$38,$FF,$38,$FF,$00,$00
    DB $FE,$FF,$C0,$FF,$C0,$FF,$F8,$FF
    DB $C0,$FF,$C0,$FF,$FE,$FF,$00,$00
    DB $FC,$FF,$C6,$FF,$C6,$FF,$FC,$FF
    DB $F8,$FF,$DC,$FF,$CE,$FF,$00,$00
    DB $18,$FF,$3C,$FF,$3C,$FF,$18,$FF
    DB $18,$FF,$00,$FF,$18,$FF,$00,$00
    DB $7C,$7C,$C6,$C6,$C0,$C0,$CE,$CE
    DB $C6,$C6,$C6,$C6,$7C,$7C,$00,$00
    DB $38,$38,$6C,$6C,$C6,$C6,$FE,$FE
    DB $C6,$C6,$C6,$C6,$C6,$C6,$00,$00
    DB $C6,$C6,$EE,$EE,$FE,$FE,$D6,$D6
    DB $C6,$C6,$C6,$C6,$C6,$C6,$00,$00
    DB $FE,$FE,$C0,$C0,$C0,$C0,$F8,$F8
    DB $C0,$C0,$C0,$C0,$FE,$FE,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $7C,$7C,$C6,$C6,$C6,$C6,$C6,$C6
    DB $C6,$C6,$C6,$C6,$7C,$7C,$00,$00
    DB $C6,$C6,$C6,$C6,$6C,$6C,$6C,$6C
    DB $6C,$6C,$38,$38,$38,$38,$00,$00
    DB $FE,$FE,$C0,$C0,$C0,$C0,$F8,$F8
    DB $C0,$C0,$C0,$C0,$FE,$FE,$00,$00
    DB $FC,$FC,$C6,$C6,$C6,$C6,$FC,$FC
    DB $F8,$F8,$DC,$DC,$CE,$CE,$00,$00
    DB $18,$18,$3C,$3C,$3C,$3C,$18,$18
    DB $18,$18,$00,$00,$18,$18,$00,$00
    DB $00,$FF,$00,$FF,$00,$FF,$00,$FF
    DB $00,$FF,$00,$FF,$00,$FF,$00,$00
    DB $FC,$FF,$C6,$FF,$C6,$FF,$FC,$FF
    DB $F8,$FF,$DC,$FF,$CE,$FF,$00,$00
    DB $FE,$FF,$C0,$FF,$C0,$FF,$F8,$FF
    DB $C0,$FF,$C0,$FF,$FE,$FF,$00,$00
    DB $FC,$FF,$30,$FF,$30,$FF,$30,$FF
    DB $30,$FF,$30,$FF,$30,$FF,$00,$00
    DB $FC,$FF,$C6,$FF,$C6,$FF,$FC,$FF
    DB $F8,$FF,$DC,$FF,$CE,$FF,$00,$00
    DB $C6,$FF,$EE,$FF,$7C,$FF,$38,$FF
    DB $70,$FF,$E0,$FF,$C0,$FF,$00,$00
    DB $F0,$FF,$C0,$FF,$C0,$FF,$C0,$FF
    DB $C0,$FF,$C0,$FF,$F0,$FF,$00,$00
    DB $38,$FF,$6C,$FF,$C6,$FF,$FE,$FF
    DB $C6,$FF,$C6,$FF,$C6,$FF,$00,$00
    DB $0F,$FF,$03,$FF,$03,$FF,$03,$FF
    DB $03,$FF,$03,$FF,$0F,$FF,$00,$00
    DB $00,$FF,$00,$FF,$00,$FF,$00,$FF
    DB $00,$FF,$00,$FF,$00,$FF,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $FC,$FC,$C6,$C6,$C6,$C6,$FC,$FC
    DB $F8,$F8,$DC,$DC,$CE,$CE,$00,$00
    DB $FE,$FE,$C0,$C0,$C0,$C0,$F8,$F8
    DB $C0,$C0,$C0,$C0,$FE,$FE,$00,$00
    DB $FC,$FC,$30,$30,$30,$30,$30,$30
    DB $30,$30,$30,$30,$30,$30,$00,$00
    DB $FC,$FC,$C6,$C6,$C6,$C6,$FC,$FC
    DB $F8,$F8,$DC,$DC,$CE,$CE,$00,$00
    DB $C6,$C6,$EE,$EE,$7C,$7C,$38,$38
    DB $70,$70,$E0,$E0,$C0,$C0,$00,$00
    DB $F0,$F0,$C0,$C0,$C0,$C0,$C0,$C0
    DB $C0,$C0,$C0,$C0,$F0,$F0,$00,$00
    DB $38,$38,$6C,$6C,$C6,$C6,$FE,$FE
    DB $C6,$C6,$C6,$C6,$C6,$C6,$00,$00
    DB $0F,$0F,$03,$03,$03,$03,$03,$03
    DB $03,$03,$03,$03,$0F,$0F,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$FF,$00,$FF,$00,$FF,$00,$FF
    DB $00,$FF,$00,$FF,$00,$FF,$00,$00
    DB $7C,$FF,$C6,$FF,$C6,$FF,$C6,$FF
    DB $D6,$FF,$DE,$FF,$7C,$FF,$00,$00
    DB $C6,$FF,$C6,$FF,$C6,$FF,$C6,$FF
    DB $C6,$FF,$C6,$FF,$7C,$FF,$00,$00
    DB $18,$FF,$18,$FF,$18,$FF,$18,$FF
    DB $18,$FF,$18,$FF,$18,$FF,$00,$00
    DB $FC,$FF,$30,$FF,$30,$FF,$30,$FF
    DB $30,$FF,$30,$FF,$30,$FF,$00,$00
    DB $00,$FF,$00,$FF,$00,$FF,$00,$FF
    DB $00,$FF,$00,$FF,$00,$FF,$00,$00
    DB $F0,$FF,$C0,$FF,$C0,$FF,$C0,$FF
    DB $C0,$FF,$C0,$FF,$F0,$FF,$00,$00
    DB $FC,$FF,$C6,$FF,$C6,$FF,$FC,$FF
    DB $C6,$FF,$C6,$FF,$FC,$FF,$00,$00
    DB $0F,$FF,$03,$FF,$03,$FF,$03,$FF
    DB $03,$FF,$03,$FF,$0F,$FF,$00,$00
    DB $00,$FF,$00,$FF,$00,$FF,$00,$FF
    DB $00,$FF,$00,$FF,$00,$FF,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $7C,$7C,$C6,$C6,$C6,$C6,$C6,$C6
    DB $D6,$D6,$DE,$DE,$7C,$7C,$00,$00
    DB $C6,$C6,$C6,$C6,$C6,$C6,$C6,$C6
    DB $C6,$C6,$C6,$C6,$7C,$7C,$00,$00
    DB $18,$18,$18,$18,$18,$18,$18,$18
    DB $18,$18,$18,$18,$18,$18,$00,$00
    DB $FC,$FC,$30,$30,$30,$30,$30,$30
    DB $30,$30,$30,$30,$30,$30,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $F0,$F0,$C0,$C0,$C0,$C0,$C0,$C0
    DB $C0,$C0,$C0,$C0,$F0,$F0,$00,$00
    DB $FC,$FC,$C6,$C6,$C6,$C6,$FC,$FC
    DB $C6,$C6,$C6,$C6,$FC,$FC,$00,$00
    DB $0F,$0F,$03,$03,$03,$03,$03,$03
    DB $03,$03,$03,$03,$0F,$0F,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $44,$44,$CA,$CA,$4A,$4A,$4A,$4A
    DB $4A,$4A,$4A,$4A,$E4,$E4,$00,$00
    DB $44,$44,$CC,$CC,$44,$44,$44,$44
    DB $44,$44,$44,$44,$EE,$EE,$00,$00
    DB $4C,$4C,$C2,$C2,$42,$42,$44,$44
    DB $48,$48,$48,$48,$EE,$EE,$00,$00
    DB $4C,$4C,$C2,$C2,$42,$42,$44,$44
    DB $42,$42,$42,$42,$EC,$EC,$00,$00
    DB $FE,$FE,$DB,$DB,$DB,$DB,$DB,$DB
    DB $DB,$DB,$DB,$DB,$DB,$DB,$00,$00
    DB $FF,$06,$FF,$06,$FF,$00,$FF,$00
    DB $FF,$06,$FF,$06,$FF,$00,$00,$00
    DB $18,$18,$24,$24,$24,$24,$18,$00
    DB $24,$24,$24,$24,$18,$18,$00,$00
    DB $18,$00,$24,$04,$24,$04,$18,$00
    DB $24,$04,$24,$04,$18,$00,$00,$00
    DB $18,$18,$24,$04,$24,$04,$18,$18
    DB $24,$20,$24,$20,$18,$18,$00,$00
    DB $18,$18,$24,$04,$24,$04,$18,$18
    DB $24,$04,$24,$04,$18,$18,$00,$00
    DB $18,$00,$24,$24,$24,$24,$18,$18
    DB $24,$04,$24,$04,$18,$00,$00,$00
    DB $18,$18,$24,$20,$24,$20,$18,$18
    DB $24,$04,$24,$04,$18,$18,$00,$00
    DB $18,$18,$24,$20,$24,$20,$18,$18
    DB $24,$24,$24,$24,$18,$18,$00,$00
    DB $18,$18,$24,$04,$24,$04,$18,$00
    DB $24,$04,$24,$04,$18,$00,$00,$00
    DB $18,$18,$24,$24,$24,$24,$18,$18
    DB $24,$24,$24,$24,$18,$18,$00,$00
    DB $18,$18,$24,$24,$24,$24,$18,$18
    DB $24,$04,$24,$04,$18,$18,$00,$00
    DB $FF,$00,$FF,$00,$FF,$00,$FF,$00
    DB $FF,$60,$FF,$60,$FF,$00,$00,$00
    DB $1C,$14,$1C,$14,$1C,$14,$1F,$13
    DB $0F,$08,$07,$07,$00,$00,$00,$00
    DB $00,$00,$40,$00,$C0,$00,$40,$00
    DB $40,$00,$E8,$00,$00,$00,$00,$00
    DB $00,$00,$40,$00,$A0,$00,$20,$00
    DB $40,$00,$E8,$00,$00,$00,$00,$00
    DB $00,$00,$E0,$00,$20,$00,$40,$00
    DB $20,$00,$C8,$00,$00,$00,$00,$00
    DB $FF,$00,$FF,$00,$FF,$00,$FF,$00
    DB $FF,$00,$FF,$00,$FF,$00,$00,$00
    DB $E7,$F8,$E7,$F8,$E7,$F8,$E7,$F8
    DB $E7,$F8,$E7,$F8,$E7,$F8,$00,$00
    DB $E6,$F8,$E6,$F8,$FF,$FF,$E7,$F8
    DB $FF,$FF,$E7,$F8,$E7,$F8,$00,$00
    DB $00,$00,$00,$00,$FF,$FF,$FF,$00
    DB $FF,$FF,$FF,$00,$FF,$00,$00,$00
    DB $1C,$14,$1C,$14,$FC,$E4,$F8,$08
    DB $FC,$E4,$FC,$14,$FC,$14,$00,$00
    DB $FC,$14,$FC,$14,$FC,$14,$FC,$14
    DB $FC,$14,$FC,$14,$FC,$14,$00,$00
    DB $1C,$14,$1C,$14,$1C,$14,$1D,$15
    DB $1D,$15,$1D,$15,$1D,$15,$00,$00
    DB $1D,$15,$1C,$14,$1C,$14,$1C,$14
    DB $1C,$14,$1C,$14,$1C,$14,$00,$00
    DB $00,$00,$00,$00,$00,$00,$DD,$DD
    DB $15,$15,$59,$59,$55,$55,$00,$00
    DB $D5,$D5,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$D9,$D9
    DB $55,$55,$D5,$D5,$55,$55,$00,$00
    DB $59,$59,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $1C,$14,$1C,$14,$1C,$14,$DC,$D4
    DB $1C,$14,$9C,$94,$1C,$14,$00,$00
    DB $DC,$D4,$1C,$14,$1C,$14,$1C,$14
    DB $1C,$14,$1C,$14,$1C,$14,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$F7,$F7,$FF,$08
    DB $FF,$E3,$1C,$14,$1C,$14,$00,$00
    DB $1C,$14,$1C,$14,$FF,$E3,$FF,$08
    DB $FF,$F7,$FF,$00,$FF,$00,$00,$00
    DB $1C,$14,$1C,$14,$1C,$14,$FC,$E4
    DB $F8,$08,$FC,$E4,$1C,$14,$00,$00
    DB $00,$00,$00,$00,$00,$00,$FF,$FF
    DB $FF,$00,$FF,$FF,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$1F,$00,$20
    DB $00,$1F,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$FF,$00,$00
    DB $00,$FF,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$FF,$00,$00
    DB $00,$FF,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$FF,$00,$00
    DB $00,$FF,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$F0,$00,$08
    DB $00,$F0,$00,$00,$00,$00,$00,$00
    DB $18,$18,$2C,$34,$62,$5E,$76,$6E
    DB $24,$3C,$24,$3C,$18,$18,$00,$00
    DB $00,$00,$00,$00,$00,$00,$10,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$30,$30,$30,$30,$00,$00
sGameplayTilesMEnd::

sGameplayTilesC::
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$7E,$81,$00,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$7E,$81,$00,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$7E,$81,$00,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$7E,$81,$00,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$7E,$81,$00,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$7E,$81,$00,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$7E,$81,$00,$FF,$00,$00
    DB $01,$FF,$7F,$81,$7F,$81,$7F,$81
    DB $7F,$81,$7F,$81,$7F,$FF,$00,$00
    DB $01,$FF,$7F,$81,$7F,$81,$7F,$81
    DB $7F,$81,$7F,$81,$7F,$FF,$00,$00
    DB $01,$FF,$7F,$81,$7F,$81,$7F,$81
    DB $7F,$81,$7F,$81,$7F,$FF,$00,$00
    DB $01,$FF,$7F,$81,$7F,$81,$7F,$81
    DB $7F,$81,$7F,$81,$7F,$FF,$00,$00
    DB $01,$FF,$7F,$81,$7F,$81,$7F,$81
    DB $7F,$81,$7F,$81,$7F,$FF,$00,$00
    DB $01,$FF,$7F,$81,$7F,$81,$7F,$81
    DB $7F,$81,$7F,$81,$7F,$FF,$00,$00
    DB $01,$FF,$7F,$81,$7F,$81,$7F,$81
    DB $7F,$81,$7F,$81,$7F,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$FE,$AB,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$FE,$AB,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$FE,$AB,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$FE,$AB,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$FE,$AB,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$FE,$AB,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $7E,$81,$FE,$AB,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $FE,$AB,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $FE,$AB,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $FE,$AB,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $FE,$AB,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $FE,$AB,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $FE,$AB,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$7E,$81
    DB $FE,$AB,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$FE,$AB
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$FE,$AB
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$FE,$AB
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$FE,$AB
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$FE,$AB
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$FE,$AB
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$7E,$81,$FE,$AB
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$FE,$AB,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$FE,$AB,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$FE,$AB,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$FE,$AB,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$FE,$AB,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$FE,$AB,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$7E,$81,$FE,$AB,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$FE,$AB,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$FE,$AB,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$FE,$AB,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$FE,$AB,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$FE,$AB,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$FE,$AB,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $00,$FF,$FE,$AB,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $AA,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $AA,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $AA,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $AA,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $AA,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $AA,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $AA,$FF,$FF,$FF,$FF,$FF,$FF,$FF
    DB $FF,$FF,$FF,$FF,$FF,$FF,$00,$00
    DB $FF,$FF,$FF,$99,$FF,$BD,$FF,$BD
    DB $FF,$BD,$FF,$99,$FF,$FF,$00,$00
    DB $00,$FF,$66,$99,$42,$BD,$42,$BD
    DB $42,$BD,$66,$99,$00,$FF,$00,$00
    DB $FF,$FF,$AA,$FF,$00,$FF,$AA,$55
    DB $00,$FF,$55,$FF,$FF,$FF,$00,$00
    DB $DB,$E7,$A5,$C3,$42,$81,$91,$00
    DB $42,$81,$A5,$C3,$DB,$E7,$00,$00
    DB $00,$00,$00,$00,$00,$00,$10,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $E6,$F8,$E6,$F8,$E6,$F8,$E6,$F8
    DB $E6,$F8,$E6,$F8,$E6,$F8,$00,$00
    DB $E6,$F8,$E6,$F8,$FF,$FF,$E7,$F8
    DB $FF,$FF,$E6,$F8,$E6,$F8,$00,$00
    DB $1C,$14,$1C,$14,$1C,$14,$1C,$14
    DB $1C,$14,$1C,$14,$1C,$14,$00,$00
    DB $00,$00,$00,$00,$FF,$FF,$FF,$00
    DB $FF,$FF,$00,$00,$00,$00,$00,$00
    DB $1C,$14,$1C,$14,$FC,$E4,$F8,$08
    DB $FC,$E4,$1C,$14,$1C,$14,$00,$00
    DB $00,$00,$00,$00,$00,$FF,$FF,$FF
    DB $00,$FF,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$FF,$FF,$00,$FF
    DB $FF,$FF,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$FF,$7F,$80
    DB $7F,$80,$00,$FF,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$FF,$FE,$01
    DB $FE,$01,$00,$FF,$00,$00,$00,$00
    DB $00,$3F,$1F,$20,$1F,$20,$01,$3E
    DB $01,$02,$01,$02,$01,$02,$00,$03
    DB $00,$C0,$80,$40,$80,$40,$80,$40
    DB $80,$7C,$F8,$04,$F8,$04,$00,$FC
    DB $00,$03,$01,$02,$01,$02,$01,$02
    DB $01,$3E,$1F,$20,$1F,$20,$00,$3F
    DB $00,$FC,$F8,$04,$F8,$04,$80,$7C
    DB $80,$40,$80,$40,$80,$40,$00,$C0
    DB $00,$3F,$1F,$20,$1F,$20,$00,$3F
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$FC,$F8,$04,$F8,$04,$18,$E4
    DB $18,$24,$18,$24,$18,$24,$00,$3C
    DB $00,$3F,$1F,$20,$1F,$20,$18,$27
    DB $18,$24,$18,$24,$18,$24,$00,$3C
    DB $00,$FC,$F8,$04,$F8,$04,$00,$FC
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$0F,$07,$08,$07,$08,$07,$08
    DB $07,$08,$07,$08,$07,$08,$00,$0F
    DB $00,$F0,$E0,$10,$E0,$10,$E0,$10
    DB $E0,$10,$E0,$10,$E0,$10,$00,$F0
    DB $00,$3F,$1F,$20,$1F,$20,$01,$3E
    DB $01,$02,$01,$02,$01,$02,$00,$03
    DB $00,$FC,$F8,$04,$F8,$04,$80,$7C
    DB $80,$40,$80,$40,$80,$40,$00,$C0
    DB $00,$75,$00,$47,$00,$55,$00,$55
    DB $00,$75,$00,$00,$00,$3E,$00,$00
    DB $00,$75,$00,$47,$00,$55,$00,$55
    DB $00,$75,$00,$00,$00,$36,$00,$00
    DB $00,$75,$00,$47,$00,$55,$00,$55
    DB $00,$75,$00,$00,$00,$2A,$00,$00
    DB $00,$00,$00,$3E,$00,$7F,$00,$6B
    DB $00,$3E,$00,$2A,$00,$00,$00,$00
    DB $00,$00,$00,$FF,$20,$81,$76,$81
    DB $20,$81,$00,$FF,$00,$00,$00,$00
    DB $00,$26,$00,$55,$00,$76,$00,$55
    DB $00,$55,$00,$00,$00,$3E,$00,$00
    DB $00,$26,$00,$55,$00,$76,$00,$55
    DB $00,$55,$00,$00,$00,$36,$00,$00
    DB $00,$00,$00,$FF,$20,$81,$76,$81
    DB $20,$81,$00,$FF,$00,$00,$00,$00
    DB $02,$70,$02,$40,$02,$60,$07,$40
    DB $07,$40,$02,$00,$00,$1F,$00,$00
    DB $02,$70,$02,$40,$02,$70,$07,$10
    DB $07,$70,$02,$00,$00,$1F,$00,$00
    DB $02,$50,$02,$50,$02,$70,$07,$50
    DB $07,$50,$02,$00,$00,$1F,$00,$00
    DB $02,$40,$02,$40,$02,$40,$07,$40
    DB $07,$70,$02,$00,$00,$1F,$00,$00
    DB $02,$50,$02,$50,$02,$20,$07,$50
    DB $07,$50,$02,$00,$00,$1F,$00,$00
    DB $00,$3E,$02,$61,$04,$51,$24,$49
    DB $00,$45,$30,$43,$00,$3E,$00,$00
    DB $00,$00,$00,$62,$00,$15,$00,$25
    DB $00,$45,$00,$72,$00,$00,$00,$00
    DB $00,$FF,$00,$FF,$00,$FF,$00,$FF
    DB $00,$FF,$00,$FF,$00,$FF,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $7C,$FF,$C6,$FF,$C0,$FF,$CE,$FF
    DB $C6,$FF,$C6,$FF,$7C,$FF,$00,$00
    DB $38,$FF,$6C,$FF,$C6,$FF,$FE,$FF
    DB $C6,$FF,$C6,$FF,$C6,$FF,$00,$00
    DB $C6,$FF,$EE,$FF,$FE,$FF,$D6,$FF
    DB $C6,$FF,$C6,$FF,$C6,$FF,$00,$00
    DB $FE,$FF,$C0,$FF,$C0,$FF,$F8,$FF
    DB $C0,$FF,$C0,$FF,$FE,$FF,$00,$00
    DB $00,$FF,$00,$FF,$00,$FF,$00,$FF
    DB $00,$FF,$00,$FF,$00,$FF,$00,$00
    DB $7C,$FF,$C6,$FF,$C6,$FF,$C6,$FF
    DB $C6,$FF,$C6,$FF,$7C,$FF,$00,$00
    DB $C6,$FF,$C6,$FF,$6C,$FF,$6C,$FF
    DB $6C,$FF,$38,$FF,$38,$FF,$00,$00
    DB $FE,$FF,$C0,$FF,$C0,$FF,$F8,$FF
    DB $C0,$FF,$C0,$FF,$FE,$FF,$00,$00
    DB $FC,$FF,$C6,$FF,$C6,$FF,$FC,$FF
    DB $F8,$FF,$DC,$FF,$CE,$FF,$00,$00
    DB $18,$FF,$3C,$FF,$3C,$FF,$18,$FF
    DB $18,$FF,$00,$FF,$18,$FF,$00,$00
    DB $7C,$7C,$C6,$C6,$C0,$C0,$CE,$CE
    DB $C6,$C6,$C6,$C6,$7C,$7C,$00,$00
    DB $38,$38,$6C,$6C,$C6,$C6,$FE,$FE
    DB $C6,$C6,$C6,$C6,$C6,$C6,$00,$00
    DB $C6,$C6,$EE,$EE,$FE,$FE,$D6,$D6
    DB $C6,$C6,$C6,$C6,$C6,$C6,$00,$00
    DB $FE,$FE,$C0,$C0,$C0,$C0,$F8,$F8
    DB $C0,$C0,$C0,$C0,$FE,$FE,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $7C,$7C,$C6,$C6,$C6,$C6,$C6,$C6
    DB $C6,$C6,$C6,$C6,$7C,$7C,$00,$00
    DB $C6,$C6,$C6,$C6,$6C,$6C,$6C,$6C
    DB $6C,$6C,$38,$38,$38,$38,$00,$00
    DB $FE,$FE,$C0,$C0,$C0,$C0,$F8,$F8
    DB $C0,$C0,$C0,$C0,$FE,$FE,$00,$00
    DB $FC,$FC,$C6,$C6,$C6,$C6,$FC,$FC
    DB $F8,$F8,$DC,$DC,$CE,$CE,$00,$00
    DB $18,$18,$3C,$3C,$3C,$3C,$18,$18
    DB $18,$18,$00,$00,$18,$18,$00,$00
    DB $00,$FF,$00,$FF,$00,$FF,$00,$FF
    DB $00,$FF,$00,$FF,$00,$FF,$00,$00
    DB $FC,$FF,$C6,$FF,$C6,$FF,$FC,$FF
    DB $F8,$FF,$DC,$FF,$CE,$FF,$00,$00
    DB $FE,$FF,$C0,$FF,$C0,$FF,$F8,$FF
    DB $C0,$FF,$C0,$FF,$FE,$FF,$00,$00
    DB $FC,$FF,$30,$FF,$30,$FF,$30,$FF
    DB $30,$FF,$30,$FF,$30,$FF,$00,$00
    DB $FC,$FF,$C6,$FF,$C6,$FF,$FC,$FF
    DB $F8,$FF,$DC,$FF,$CE,$FF,$00,$00
    DB $C6,$FF,$EE,$FF,$7C,$FF,$38,$FF
    DB $70,$FF,$E0,$FF,$C0,$FF,$00,$00
    DB $F0,$FF,$C0,$FF,$C0,$FF,$C0,$FF
    DB $C0,$FF,$C0,$FF,$F0,$FF,$00,$00
    DB $38,$FF,$6C,$FF,$C6,$FF,$FE,$FF
    DB $C6,$FF,$C6,$FF,$C6,$FF,$00,$00
    DB $0F,$FF,$03,$FF,$03,$FF,$03,$FF
    DB $03,$FF,$03,$FF,$0F,$FF,$00,$00
    DB $00,$FF,$00,$FF,$00,$FF,$00,$FF
    DB $00,$FF,$00,$FF,$00,$FF,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $FC,$FC,$C6,$C6,$C6,$C6,$FC,$FC
    DB $F8,$F8,$DC,$DC,$CE,$CE,$00,$00
    DB $FE,$FE,$C0,$C0,$C0,$C0,$F8,$F8
    DB $C0,$C0,$C0,$C0,$FE,$FE,$00,$00
    DB $FC,$FC,$30,$30,$30,$30,$30,$30
    DB $30,$30,$30,$30,$30,$30,$00,$00
    DB $FC,$FC,$C6,$C6,$C6,$C6,$FC,$FC
    DB $F8,$F8,$DC,$DC,$CE,$CE,$00,$00
    DB $C6,$C6,$EE,$EE,$7C,$7C,$38,$38
    DB $70,$70,$E0,$E0,$C0,$C0,$00,$00
    DB $F0,$F0,$C0,$C0,$C0,$C0,$C0,$C0
    DB $C0,$C0,$C0,$C0,$F0,$F0,$00,$00
    DB $38,$38,$6C,$6C,$C6,$C6,$FE,$FE
    DB $C6,$C6,$C6,$C6,$C6,$C6,$00,$00
    DB $0F,$0F,$03,$03,$03,$03,$03,$03
    DB $03,$03,$03,$03,$0F,$0F,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$FF,$00,$FF,$00,$FF,$00,$FF
    DB $00,$FF,$00,$FF,$00,$FF,$00,$00
    DB $7C,$FF,$C6,$FF,$C6,$FF,$C6,$FF
    DB $D6,$FF,$DE,$FF,$7C,$FF,$00,$00
    DB $C6,$FF,$C6,$FF,$C6,$FF,$C6,$FF
    DB $C6,$FF,$C6,$FF,$7C,$FF,$00,$00
    DB $18,$FF,$18,$FF,$18,$FF,$18,$FF
    DB $18,$FF,$18,$FF,$18,$FF,$00,$00
    DB $FC,$FF,$30,$FF,$30,$FF,$30,$FF
    DB $30,$FF,$30,$FF,$30,$FF,$00,$00
    DB $00,$FF,$00,$FF,$00,$FF,$00,$FF
    DB $00,$FF,$00,$FF,$00,$FF,$00,$00
    DB $F0,$FF,$C0,$FF,$C0,$FF,$C0,$FF
    DB $C0,$FF,$C0,$FF,$F0,$FF,$00,$00
    DB $FC,$FF,$C6,$FF,$C6,$FF,$FC,$FF
    DB $C6,$FF,$C6,$FF,$FC,$FF,$00,$00
    DB $0F,$FF,$03,$FF,$03,$FF,$03,$FF
    DB $03,$FF,$03,$FF,$0F,$FF,$00,$00
    DB $00,$FF,$00,$FF,$00,$FF,$00,$FF
    DB $00,$FF,$00,$FF,$00,$FF,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $7C,$7C,$C6,$C6,$C6,$C6,$C6,$C6
    DB $D6,$D6,$DE,$DE,$7C,$7C,$00,$00
    DB $C6,$C6,$C6,$C6,$C6,$C6,$C6,$C6
    DB $C6,$C6,$C6,$C6,$7C,$7C,$00,$00
    DB $18,$18,$18,$18,$18,$18,$18,$18
    DB $18,$18,$18,$18,$18,$18,$00,$00
    DB $FC,$FC,$30,$30,$30,$30,$30,$30
    DB $30,$30,$30,$30,$30,$30,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $F0,$F0,$C0,$C0,$C0,$C0,$C0,$C0
    DB $C0,$C0,$C0,$C0,$F0,$F0,$00,$00
    DB $FC,$FC,$C6,$C6,$C6,$C6,$FC,$FC
    DB $C6,$C6,$C6,$C6,$FC,$FC,$00,$00
    DB $0F,$0F,$03,$03,$03,$03,$03,$03
    DB $03,$03,$03,$03,$0F,$0F,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $44,$44,$CA,$CA,$4A,$4A,$4A,$4A
    DB $4A,$4A,$4A,$4A,$E4,$E4,$00,$00
    DB $44,$44,$CC,$CC,$44,$44,$44,$44
    DB $44,$44,$44,$44,$EE,$EE,$00,$00
    DB $4C,$4C,$C2,$C2,$42,$42,$44,$44
    DB $48,$48,$48,$48,$EE,$EE,$00,$00
    DB $4C,$4C,$C2,$C2,$42,$42,$44,$44
    DB $42,$42,$42,$42,$EC,$EC,$00,$00
    DB $FE,$FE,$DB,$DB,$DB,$DB,$DB,$DB
    DB $DB,$DB,$DB,$DB,$DB,$DB,$00,$00
    DB $FF,$06,$FF,$06,$FF,$00,$FF,$00
    DB $FF,$06,$FF,$06,$FF,$00,$00,$00
    DB $18,$18,$24,$24,$24,$24,$18,$00
    DB $24,$24,$24,$24,$18,$18,$00,$00
    DB $18,$00,$24,$04,$24,$04,$18,$00
    DB $24,$04,$24,$04,$18,$00,$00,$00
    DB $18,$18,$24,$04,$24,$04,$18,$18
    DB $24,$20,$24,$20,$18,$18,$00,$00
    DB $18,$18,$24,$04,$24,$04,$18,$18
    DB $24,$04,$24,$04,$18,$18,$00,$00
    DB $18,$00,$24,$24,$24,$24,$18,$18
    DB $24,$04,$24,$04,$18,$00,$00,$00
    DB $18,$18,$24,$20,$24,$20,$18,$18
    DB $24,$04,$24,$04,$18,$18,$00,$00
    DB $18,$18,$24,$20,$24,$20,$18,$18
    DB $24,$24,$24,$24,$18,$18,$00,$00
    DB $18,$18,$24,$04,$24,$04,$18,$00
    DB $24,$04,$24,$04,$18,$00,$00,$00
    DB $18,$18,$24,$24,$24,$24,$18,$18
    DB $24,$24,$24,$24,$18,$18,$00,$00
    DB $18,$18,$24,$24,$24,$24,$18,$18
    DB $24,$04,$24,$04,$18,$18,$00,$00
    DB $FF,$00,$FF,$00,$FF,$00,$FF,$00
    DB $FF,$60,$FF,$60,$FF,$00,$00,$00
    DB $1C,$14,$1C,$14,$1C,$14,$1F,$13
    DB $0F,$08,$07,$07,$00,$00,$00,$00
    DB $00,$00,$40,$00,$C0,$00,$40,$00
    DB $40,$00,$E8,$00,$00,$00,$00,$00
    DB $00,$00,$40,$00,$A0,$00,$20,$00
    DB $40,$00,$E8,$00,$00,$00,$00,$00
    DB $00,$00,$E0,$00,$20,$00,$40,$00
    DB $20,$00,$C8,$00,$00,$00,$00,$00
    DB $FF,$00,$FF,$00,$FF,$00,$FF,$00
    DB $FF,$00,$FF,$00,$FF,$00,$00,$00
    DB $E7,$F8,$E7,$F8,$E7,$F8,$E7,$F8
    DB $E7,$F8,$E7,$F8,$E7,$F8,$00,$00
    DB $E6,$F8,$E6,$F8,$FF,$FF,$E7,$F8
    DB $FF,$FF,$E7,$F8,$E7,$F8,$00,$00
    DB $00,$00,$00,$00,$FF,$FF,$FF,$00
    DB $FF,$FF,$FF,$00,$FF,$00,$00,$00
    DB $1C,$14,$1C,$14,$FC,$E4,$F8,$08
    DB $FC,$E4,$FC,$14,$FC,$14,$00,$00
    DB $FC,$14,$FC,$14,$FC,$14,$FC,$14
    DB $FC,$14,$FC,$14,$FC,$14,$00,$00
    DB $1C,$14,$1C,$14,$1C,$14,$1D,$15
    DB $1D,$15,$1D,$15,$1D,$15,$00,$00
    DB $1D,$15,$1C,$14,$1C,$14,$1C,$14
    DB $1C,$14,$1C,$14,$1C,$14,$00,$00
    DB $00,$00,$00,$00,$00,$00,$DD,$DD
    DB $15,$15,$59,$59,$55,$55,$00,$00
    DB $D5,$D5,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$D9,$D9
    DB $55,$55,$D5,$D5,$55,$55,$00,$00
    DB $59,$59,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $1C,$14,$1C,$14,$1C,$14,$DC,$D4
    DB $1C,$14,$9C,$94,$1C,$14,$00,$00
    DB $DC,$D4,$1C,$14,$1C,$14,$1C,$14
    DB $1C,$14,$1C,$14,$1C,$14,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$F7,$F7,$FF,$08
    DB $FF,$E3,$1C,$14,$1C,$14,$00,$00
    DB $1C,$14,$1C,$14,$FF,$E3,$FF,$08
    DB $FF,$F7,$FF,$00,$FF,$00,$00,$00
    DB $1C,$14,$1C,$14,$1C,$14,$FC,$E4
    DB $F8,$08,$FC,$E4,$1C,$14,$00,$00
    DB $00,$00,$00,$00,$00,$00,$FF,$FF
    DB $FF,$00,$FF,$FF,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$1F,$00,$20
    DB $00,$1F,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$FF,$00,$00
    DB $00,$FF,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$FF,$00,$00
    DB $00,$FF,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$FF,$00,$00
    DB $00,$FF,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$F0,$00,$08
    DB $00,$F0,$00,$00,$00,$00,$00,$00
    DB $18,$18,$2C,$34,$62,$5E,$76,$6E
    DB $24,$3C,$24,$3C,$18,$18,$00,$00
    DB $00,$00,$00,$00,$00,$00,$10,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$00,$00,$00,$00,$00,$00
    DB $00,$00,$30,$30,$30,$30,$00,$00
sGameplayTilesCEnd::

ENDC
